# -*- coding: utf-8 -*-
import requests
import unittest

from requests_toolbelt.cookies.forgetful import ForgetfulCookieJar
from . import get_betamax


class TestForgetfulCookieJar(unittest.TestCase):

    def setUp(self):
        self.session = requests.Session()
        self.session.cookies = ForgetfulCookieJar()
        self.recorder = get_betamax(self.session)

    def test_cookies_are_ignored(self):
        with self.recorder.use_cassette('http2bin_cookies'):
            url = 'https://httpbin.org/cookies/set'
            cookies = {
                'cookie0': 'value0',
            }
            r = self.session.request(
                'GET', url,
                 params=cookies
                )
            assert 'cookie0' not in self.session.cookies



"""Collection of tests for utils.dump.

The dump utility module only has two public attributes:

- dump_response
- dump_all

This module, however, tests many of the private implementation details since
those public functions just wrap them and testing the public functions will be
very complex and high-level.
"""
from requests_toolbelt._compat import HTTPHeaderDict
from requests_toolbelt.utils import dump

try:
    from unittest import mock
except ImportError:
    import mock
import pytest
import requests

from . import get_betamax

HTTP_1_1 = 11
HTTP_1_0 = 10
HTTP_0_9 = 9
HTTP_UNKNOWN = 5000


class TestSimplePrivateFunctions(object):

    """Excercise simple private functions in one logical place."""

    def test_coerce_to_bytes_skips_byte_strings(self):
        """Show that _coerce_to_bytes skips bytes input."""
        bytestr = b'some bytes'
        assert dump._coerce_to_bytes(bytestr) is bytestr

    def test_coerce_to_bytes_converts_text(self):
        """Show that _coerce_to_bytes handles text input."""
        bytestr = b'some bytes'
        text = bytestr.decode('utf-8')
        assert dump._coerce_to_bytes(text) == bytestr

    def test_format_header(self):
        """Prove that _format_header correctly formats bytes input."""
        header = b'Connection'
        value = b'close'
        expected = b'Connection: close\r\n'
        assert dump._format_header(header, value) == expected

    def test_format_header_handles_unicode(self):
        """Prove that _format_header correctly formats text input."""
        header = b'Connection'.decode('utf-8')
        value = b'close'.decode('utf-8')
        expected = b'Connection: close\r\n'
        assert dump._format_header(header, value) == expected

    def test_build_request_path(self):
        """Show we get the right request path for a normal request."""
        path, _ = dump._build_request_path(
            'https://example.com/foo/bar', {}
        )
        assert path == b'/foo/bar'

    def test_build_request_path_with_query_string(self):
        """Show we include query strings appropriately."""
        path, _ = dump._build_request_path(
            'https://example.com/foo/bar?query=data', {}
        )
        assert path == b'/foo/bar?query=data'

    def test_build_request_path_with_proxy_info(self):
        """Show that we defer to the proxy request_path info."""
        path, _ = dump._build_request_path(
            'https://example.com/', {
                'request_path': b'https://example.com/test'
            }
        )
        assert path == b'https://example.com/test'


class RequestResponseMixin(object):

    """Mix-in for test classes needing mocked requests and responses."""

    response_spec = [
        'connection',
        'content',
        'raw',
        'reason',
        'request',
        'url',
    ]

    request_spec = [
        'body',
        'headers',
        'method',
        'url',
    ]

    httpresponse_spec = [
        'headers',
        'reason',
        'status',
        'version',
    ]

    adapter_spec = [
        'proxy_manager',
    ]

    @pytest.fixture(autouse=True)
    def set_up(self):
        """xUnit style autoused fixture creating mocks."""
        self.response = mock.Mock(spec=self.response_spec)
        self.request = mock.Mock(spec=self.request_spec)
        self.httpresponse = mock.Mock(spec=self.httpresponse_spec)
        self.adapter = mock.Mock(spec=self.adapter_spec)

        self.response.connection = self.adapter
        self.response.request = self.request
        self.response.raw = self.httpresponse

    def configure_response(self, content=b'', proxy_manager=None, url=None,
                           reason=b''):
        """Helper function to configure a mocked response."""
        self.adapter.proxy_manager = proxy_manager or {}
        self.response.content = content
        self.response.url = url
        self.response.reason = reason

    def configure_request(self, body=b'', headers=None, method=None,
                          url=None):
        """Helper function to configure a mocked request."""
        self.request.body = body
        self.request.headers = headers or {}
        self.request.method = method
        self.request.url = url

    def configure_httpresponse(self, headers=None, reason=b'', status=200,
                               version=HTTP_1_1):
        """Helper function to configure a mocked urllib3 response."""
        self.httpresponse.headers = HTTPHeaderDict(headers or {})
        self.httpresponse.reason = reason
        self.httpresponse.status = status
        self.httpresponse.version = version


class TestResponsePrivateFunctions(RequestResponseMixin):

    """Excercise private functions using responses."""

    def test_get_proxy_information_sans_proxy(self):
        """Show no information is returned when not using a proxy."""
        self.configure_response()

        assert dump._get_proxy_information(self.response) is None

    def test_get_proxy_information_with_proxy_over_http(self):
        """Show only the request path is returned for HTTP requests.

        Using HTTP over a proxy doesn't alter anything except the request path
        of the request. The method doesn't change a dictionary with the
        request_path is the only thing that should be returned.
        """
        self.configure_response(
            proxy_manager={'http://': 'http://local.proxy:3939'},
        )
        self.configure_request(
            url='http://example.com',
            method='GET',
        )

        assert dump._get_proxy_information(self.response) == {
            'request_path': 'http://example.com'
        }

    def test_get_proxy_information_with_proxy_over_https(self):
        """Show that the request path and method are returned for HTTPS reqs.

        Using HTTPS over a proxy changes the method used and the request path.
        """
        self.configure_response(
            proxy_manager={'http://': 'http://local.proxy:3939'},
        )
        self.configure_request(
            url='https://example.com',
            method='GET',
        )

        assert dump._get_proxy_information(self.response) == {
            'method': 'CONNECT',
            'request_path': 'https://example.com'
        }

    def test_dump_request_data(self):
        """Build up the request data into a bytearray."""
        self.configure_request(
            url='http://example.com/',
            method='GET',
        )

        array = bytearray()
        prefixes = dump.PrefixSettings('request:', 'response:')
        dump._dump_request_data(
            request=self.request,
            prefixes=prefixes,
            bytearr=array,
            proxy_info={},
        )

        assert b'request:GET / HTTP/1.1\r\n' in array
        assert b'request:Host: example.com\r\n' in array

    def test_dump_non_string_request_data(self):
        """Build up the request data into a bytearray."""
        self.configure_request(
            url='http://example.com/',
            method='POST',
            body=1
        )

        array = bytearray()
        prefixes = dump.PrefixSettings('request:', 'response:')
        dump._dump_request_data(
            request=self.request,
            prefixes=prefixes,
            bytearr=array,
            proxy_info={},
        )
        assert b'request:POST / HTTP/1.1\r\n' in array
        assert b'request:Host: example.com\r\n' in array
        assert b'<< Request body is not a string-like type >>\r\n' in array

    def test_dump_request_data_with_proxy_info(self):
        """Build up the request data into a bytearray."""
        self.configure_request(
            url='http://example.com/',
            method='GET',
        )

        array = bytearray()
        prefixes = dump.PrefixSettings('request:', 'response:')
        dump._dump_request_data(
            request=self.request,
            prefixes=prefixes,
            bytearr=array,
            proxy_info={
                'request_path': b'fake-request-path',
                'method': b'CONNECT',
            },
        )

        assert b'request:CONNECT fake-request-path HTTP/1.1\r\n' in array
        assert b'request:Host: example.com\r\n' in array

    def test_dump_response_data(self):
        """Build up the response data into a bytearray."""
        self.configure_response(
            url='https://example.com/redirected',
            content=b'foobarbogus',
            reason=b'OK',
        )
        self.configure_httpresponse(
            headers={'Content-Type': 'application/json'},
            reason=b'OK',
            status=201,
        )

        array = bytearray()
        prefixes = dump.PrefixSettings('request:', 'response:')
        dump._dump_response_data(
            response=self.response,
            prefixes=prefixes,
            bytearr=array,
        )

        assert b'response:HTTP/1.1 201 OK\r\n' in array
        assert b'response:Content-Type: application/json\r\n' in array

    def test_dump_response_data_with_older_http_version(self):
        """Build up the response data into a bytearray."""
        self.configure_response(
            url='https://example.com/redirected',
            content=b'foobarbogus',
            reason=b'OK',
        )
        self.configure_httpresponse(
            headers={'Content-Type': 'application/json'},
            reason=b'OK',
            status=201,
            version=HTTP_0_9,
        )

        array = bytearray()
        prefixes = dump.PrefixSettings('request:', 'response:')
        dump._dump_response_data(
            response=self.response,
            prefixes=prefixes,
            bytearr=array,
        )

        assert b'response:HTTP/0.9 201 OK\r\n' in array
        assert b'response:Content-Type: application/json\r\n' in array

    def test_dump_response_data_with_unknown_http_version(self):
        """Build up the response data into a bytearray."""
        self.configure_response(
            url='https://example.com/redirected',
            content=b'foobarbogus',
            reason=b'OK',
        )
        self.configure_httpresponse(
            headers={'Content-Type': 'application/json'},
            reason=b'OK',
            status=201,
            version=HTTP_UNKNOWN,
        )

        array = bytearray()
        prefixes = dump.PrefixSettings('request:', 'response:')
        dump._dump_response_data(
            response=self.response,
            prefixes=prefixes,
            bytearr=array,
        )

        assert b'response:HTTP/? 201 OK\r\n' in array
        assert b'response:Content-Type: application/json\r\n' in array


class TestResponsePublicFunctions(RequestResponseMixin):

    """Excercise public functions using responses."""

    def test_dump_response_fails_without_request(self):
        """Show that a response without a request raises a ValueError."""
        del self.response.request
        assert hasattr(self.response, 'request') is False

        with pytest.raises(ValueError):
            dump.dump_response(self.response)

    def test_dump_response_uses_provided_bytearray(self):
        """Show that users providing bytearrays receive those back."""
        self.configure_request(
            url='http://example.com/',
            method='GET',
        )
        self.configure_response(
            url='https://example.com/redirected',
            content=b'foobarbogus',
            reason=b'OK',
        )
        self.configure_httpresponse(
            headers={'Content-Type': 'application/json'},
            reason=b'OK',
            status=201,
        )
        arr = bytearray()

        retarr = dump.dump_response(self.response, data_array=arr)
        assert retarr is arr


class TestDumpRealResponses(object):

    """Exercise dump utilities against real data."""

    def test_dump_response(self):
        session = requests.Session()
        recorder = get_betamax(session)
        with recorder.use_cassette('simple_get_request'):
            response = session.get('https://httpbin.org/get')

        arr = dump.dump_response(response)
        assert b'< GET /get HTTP/1.1\r\n' in arr
        assert b'< Host: httpbin.org\r\n' in arr
        # NOTE(sigmavirus24): The ? below is only because Betamax doesn't
        # preserve which HTTP version the server reports as supporting.
        # When not using Betamax, there should be a different version
        # reported.
        assert b'> HTTP/? 200 OK\r\n' in arr
        assert b'> Content-Type: application/json\r\n' in arr

    def test_dump_all(self):
        session = requests.Session()
        recorder = get_betamax(session)
        with recorder.use_cassette('redirect_request_for_dump_all'):
            response = session.get('https://httpbin.org/redirect/5')

        arr = dump.dump_all(response)
        assert b'< GET /redirect/5 HTTP/1.1\r\n' in arr
        assert b'> Location: /relative-redirect/4\r\n' in arr
        assert b'< GET /relative-redirect/4 HTTP/1.1\r\n' in arr
        assert b'> Location: /relative-redirect/3\r\n' in arr
        assert b'< GET /relative-redirect/3 HTTP/1.1\r\n' in arr
        assert b'> Location: /relative-redirect/2\r\n' in arr
        assert b'< GET /relative-redirect/2 HTTP/1.1\r\n' in arr
        assert b'> Location: /relative-redirect/1\r\n' in arr
        assert b'< GET /relative-redirect/1 HTTP/1.1\r\n' in arr
        assert b'> Location: /get\r\n' in arr
        assert b'< GET /get HTTP/1.1\r\n' in arr



# -*- coding: utf-8 -*-
import os
import sys

import betamax

sys.path.insert(0, '.')

placeholders = {
    '<IPADDR>': os.environ.get('IPADDR', '127.0.0.1'),
}

with betamax.Betamax.configure() as config:
    for placeholder, value in placeholders.items():
        config.define_cassette_placeholder(placeholder, value)



# -*- coding: utf-8 -*-
import requests
import unittest

from requests_toolbelt.adapters.fingerprint import FingerprintAdapter
from . import get_betamax


class TestFingerprintAdapter(unittest.TestCase):
    HTTP2BIN_FINGERPRINT = 'abf8683eeba8521ad2e8dc48e92a1cbea3ff8608f1417948fdad75d7b50eb264'

    def setUp(self):
        self.session = requests.Session()
        self.session.mount('https://http2bin.org', FingerprintAdapter(self.HTTP2BIN_FINGERPRINT))
        self.recorder = get_betamax(self.session)

    def test_fingerprint(self):
        with self.recorder.use_cassette('http2bin_fingerprint'):
            r = self.session.get('https://http2bin.org/get')
            assert r.status_code == 200



# -*- coding: utf-8 -*-
from requests.adapters import DEFAULT_POOLSIZE, DEFAULT_POOLBLOCK
try:
    from unittest.mock import patch
except ImportError:
    from mock import patch
from requests_toolbelt.adapters.source import SourceAddressAdapter

import pytest


@patch('requests_toolbelt.adapters.source.poolmanager')
def test_source_address_adapter_string(poolmanager):
    SourceAddressAdapter('10.10.10.10')

    poolmanager.PoolManager.assert_called_once_with(
        num_pools=DEFAULT_POOLSIZE,
        maxsize=DEFAULT_POOLSIZE,
        block=DEFAULT_POOLBLOCK,
        source_address=('10.10.10.10', 0)
    )


@patch('requests_toolbelt.adapters.source.poolmanager')
def test_source_address_adapter_tuple(poolmanager):
    SourceAddressAdapter(('10.10.10.10', 80))

    poolmanager.PoolManager.assert_called_once_with(
        num_pools=DEFAULT_POOLSIZE,
        maxsize=DEFAULT_POOLSIZE,
        block=DEFAULT_POOLBLOCK,
        source_address=('10.10.10.10', 80)
    )


@patch('requests_toolbelt.adapters.source.poolmanager')
def test_source_address_adapter_type_error(poolmanager):
    with pytest.raises(TypeError):
        SourceAddressAdapter({'10.10.10.10': 80})

    assert not poolmanager.PoolManager.called



# -*- coding: utf-8 -*-
import unittest
import io

import requests

import pytest
from requests_toolbelt.multipart.encoder import (
    CustomBytesIO, MultipartEncoder, FileFromURLWrapper, FileNotSupportedError)
from requests_toolbelt._compat import filepost
from . import get_betamax


preserve_bytes = {'preserve_exact_body_bytes': True}


class LargeFileMock(object):
    def __init__(self):
        # Let's keep track of how many bytes we've given
        self.bytes_read = 0
        # Our limit (1GB)
        self.bytes_max = 1024 * 1024 * 1024
        # Fake name
        self.name = 'fake_name.py'
        # Create a fileno attribute
        self.fileno = None

    def __len__(self):
        return self.bytes_max

    def read(self, size=None):
        if self.bytes_read >= self.bytes_max:
            return b''

        if size is None:
            length = self.bytes_max - self.bytes_read
        else:
            length = size

        length = int(length)
        length = min([length, self.bytes_max - self.bytes_read])

        self.bytes_read += length

        return b'a' * length

    def tell(self):
        return self.bytes_read


class TestCustomBytesIO(unittest.TestCase):
    def setUp(self):
        self.instance = CustomBytesIO()

    def test_writable(self):
        assert hasattr(self.instance, 'write')
        assert self.instance.write(b'example') == 7

    def test_readable(self):
        assert hasattr(self.instance, 'read')
        assert self.instance.read() == b''
        assert self.instance.read(10) == b''

    def test_can_read_after_writing_to(self):
        self.instance.write(b'example text')
        self.instance.read() == b'example text'

    def test_can_read_some_after_writing_to(self):
        self.instance.write(b'example text')
        self.instance.read(6) == b'exampl'

    def test_can_get_length(self):
        self.instance.write(b'example')
        self.instance.seek(0, 0)
        assert self.instance.len == 7

    def test_truncates_intelligently(self):
        self.instance.write(b'abcdefghijklmnopqrstuvwxyzabcd')  # 30 bytes
        assert self.instance.tell() == 30
        self.instance.seek(-10, 2)
        self.instance.smart_truncate()
        assert self.instance.len == 10
        assert self.instance.read() == b'uvwxyzabcd'
        assert self.instance.tell() == 10

    def test_accepts_encoded_strings_with_unicode(self):
        """Accepts a string with encoded unicode characters."""
        s = b'this is a unicode string: \xc3\xa9 \xc3\xa1 \xc7\xab \xc3\xb3'
        self.instance = CustomBytesIO(s)
        assert self.instance.read() == s


class TestFileFromURLWrapper(unittest.TestCase):
    def setUp(self):
        s = requests.Session()
        self.recorder = get_betamax(s)

    @pytest.mark.xfail
    def test_read_file(self):
        url = ('https://stxnext.com/static/img/logo.830ebe551641.svg')
        with self.recorder.use_cassette(
                'file_for_download', **preserve_bytes):
            self.instance = FileFromURLWrapper(url)
            assert self.instance.len == 5177
            chunk = self.instance.read(20)
            assert chunk == b'<svg xmlns="http://w'
            assert self.instance.len == 5157
            chunk = self.instance.read(0)
            assert chunk == b''
            assert self.instance.len == 5157
            chunk = self.instance.read(10)
            assert chunk == b'ww.w3.org/'
            assert self.instance.len == 5147

    @pytest.mark.xfail(strict=False)
    def test_no_content_length_header(self):
        url = (
            'https://api.github.com/repos/sigmavirus24/github3.py/releases/'
            'assets/37944'
        )
        with self.recorder.use_cassette(
                'stream_response_to_file', **preserve_bytes):
            with self.assertRaises(FileNotSupportedError) as context:
                FileFromURLWrapper(url)
            assert context.exception.__str__() == (
                'Data from provided URL https://api.github.com/repos/s'
                'igmavirus24/github3.py/releases/assets/37944 is not '
                'supported. Lack of content-length Header in requested'
                ' file response.'
            )


class TestMultipartEncoder(unittest.TestCase):
    def setUp(self):
        self.parts = [('field', 'value'), ('other_field', 'other_value')]
        self.boundary = 'this-is-a-boundary'
        self.instance = MultipartEncoder(self.parts, boundary=self.boundary)

    def test_to_string(self):
        assert self.instance.to_string() == (
            '--this-is-a-boundary\r\n'
            'Content-Disposition: form-data; name="field"\r\n\r\n'
            'value\r\n'
            '--this-is-a-boundary\r\n'
            'Content-Disposition: form-data; name="other_field"\r\n\r\n'
            'other_value\r\n'
            '--this-is-a-boundary--\r\n'
        ).encode()

    def test_content_type(self):
        expected = 'multipart/form-data; boundary=this-is-a-boundary'
        assert self.instance.content_type == expected

    def test_encodes_data_the_same(self):
        encoded = filepost.encode_multipart_formdata(self.parts,
                                                     self.boundary)[0]
        assert encoded == self.instance.read()

    def test_streams_its_data(self):
        large_file = LargeFileMock()
        parts = {'some field': 'value',
                 'some file': large_file,
                 }
        encoder = MultipartEncoder(parts)
        total_size = encoder.len
        read_size = 1024 * 1024 * 128
        already_read = 0
        while True:
            read = encoder.read(read_size)
            already_read += len(read)
            if not read:
                break

        assert encoder._buffer.tell() <= read_size
        assert already_read == total_size

    def test_length_is_correct(self):
        encoded = filepost.encode_multipart_formdata(self.parts,
                                                     self.boundary)[0]
        assert len(encoded) == self.instance.len

    def test_encodes_with_readable_data(self):
        s = io.BytesIO(b'value')
        m = MultipartEncoder([('field', s)], boundary=self.boundary)
        assert m.read() == (
            '--this-is-a-boundary\r\n'
            'Content-Disposition: form-data; name="field"\r\n\r\n'
            'value\r\n'
            '--this-is-a-boundary--\r\n'
        ).encode()

    def test_reads_open_file_objects(self):
        with open('setup.py', 'rb') as fd:
            m = MultipartEncoder([('field', 'foo'), ('file', fd)])
            assert m.read() is not None

    @pytest.mark.xfail
    def test_reads_file_from_url_wrapper(self):
        s = requests.Session()
        recorder = get_betamax(s)
        url = ('https://stxnext.com/static/img/logo.830ebe551641.svg')
        with recorder.use_cassette(
                'file_for_download'):
            m = MultipartEncoder(
                [('field', 'foo'), ('file', FileFromURLWrapper(url))])
        assert m.read() is not None

    def test_reads_open_file_objects_with_a_specified_filename(self):
        with open('setup.py', 'rb') as fd:
            m = MultipartEncoder(
                [('field', 'foo'), ('file', ('filename', fd, 'text/plain'))]
                )
            assert m.read() is not None

    def test_reads_open_file_objects_using_to_string(self):
        with open('setup.py', 'rb') as fd:
            m = MultipartEncoder([('field', 'foo'), ('file', fd)])
            assert m.to_string() is not None

    def test_handles_encoded_unicode_strings(self):
        m = MultipartEncoder([
            ('field',
             b'this is a unicode string: \xc3\xa9 \xc3\xa1 \xc7\xab \xc3\xb3')
        ])
        assert m.read() is not None

    def test_handles_uncode_strings(self):
        s = b'this is a unicode string: \xc3\xa9 \xc3\xa1 \xc7\xab \xc3\xb3'
        m = MultipartEncoder([
            ('field', s.decode('utf-8'))
        ])
        assert m.read() is not None

    def test_regresion_1(self):
        """Ensure issue #31 doesn't ever happen again."""
        fields = {
            "test": "t" * 100
        }

        for x in range(30):
            fields['f%d' % x] = (
                'test', open('tests/test_multipart_encoder.py', 'rb')
                )

        m = MultipartEncoder(fields=fields)
        total_size = m.len

        blocksize = 8192
        read_so_far = 0

        while True:
            data = m.read(blocksize)
            if not data:
                break
            read_so_far += len(data)

        assert read_so_far == total_size

    def test_regression_2(self):
        """Ensure issue #31 doesn't ever happen again."""
        fields = {
            "test": "t" * 8100
        }

        m = MultipartEncoder(fields=fields)
        total_size = m.len

        blocksize = 8192
        read_so_far = 0

        while True:
            data = m.read(blocksize)
            if not data:
                break
            read_so_far += len(data)

        assert read_so_far == total_size

    def test_handles_empty_unicode_values(self):
        """Verify that the Encoder can handle empty unicode strings.

        See https://github.com/requests/toolbelt/issues/46 for
        more context.
        """
        fields = [(b'test'.decode('utf-8'), b''.decode('utf-8'))]
        m = MultipartEncoder(fields=fields)
        assert len(m.read()) > 0

    def test_accepts_custom_content_type(self):
        """Verify that the Encoder handles custom content-types.

        See https://github.com/requests/toolbelt/issues/52
        """
        fields = [
            (b'test'.decode('utf-8'), (b'filename'.decode('utf-8'),
                                       b'filecontent',
                                       b'application/json'.decode('utf-8')))
        ]
        m = MultipartEncoder(fields=fields)
        output = m.read().decode('utf-8')
        assert output.index('Content-Type: application/json\r\n') > 0

    def test_accepts_custom_headers(self):
        """Verify that the Encoder handles custom headers.

        See https://github.com/requests/toolbelt/issues/52
        """
        fields = [
            (b'test'.decode('utf-8'), (b'filename'.decode('utf-8'),
                                       b'filecontent',
                                       b'application/json'.decode('utf-8'),
                                       {'X-My-Header': 'my-value'}))
        ]
        m = MultipartEncoder(fields=fields)
        output = m.read().decode('utf-8')
        assert output.index('X-My-Header: my-value\r\n') > 0

    def test_no_parts(self):
        fields = []
        boundary = '--90967316f8404798963cce746a4f4ef9'
        m = MultipartEncoder(fields=fields, boundary=boundary)
        output = m.read().decode('utf-8')
        assert output == '----90967316f8404798963cce746a4f4ef9--\r\n'

if __name__ == '__main__':
    unittest.main()



"""Tests for the utils module."""
import io
import os
import os.path
import shutil
import tempfile

import requests
from requests_toolbelt.downloadutils import stream
from requests_toolbelt.downloadutils import tee
try:
    from unittest import mock
except ImportError:
    import mock
import pytest

from . import get_betamax


preserve_bytes = {'preserve_exact_body_bytes': True}


def test_get_download_file_path_uses_content_disposition():
    s = requests.Session()
    recorder = get_betamax(s)
    url = ('https://api.github.com/repos/sigmavirus24/github3.py/releases/'
           'assets/37944')
    filename = 'github3.py-0.7.1-py2.py3-none-any.whl'
    with recorder.use_cassette('stream_response_to_file', **preserve_bytes):
        r = s.get(url, headers={'Accept': 'application/octet-stream'})
        path = stream.get_download_file_path(r, None)
        r.close()
        assert path == filename

def test_get_download_file_path_directory():
    s = requests.Session()
    recorder = get_betamax(s)
    url = ('https://api.github.com/repos/sigmavirus24/github3.py/releases/'
           'assets/37944')
    filename = 'github3.py-0.7.1-py2.py3-none-any.whl'
    with recorder.use_cassette('stream_response_to_file', **preserve_bytes):
        r = s.get(url, headers={'Accept': 'application/octet-stream'})
        path = stream.get_download_file_path(r, tempfile.tempdir)
        r.close()
        assert path == os.path.join(tempfile.tempdir, filename)


def test_get_download_file_path_specific_file():
    s = requests.Session()
    recorder = get_betamax(s)
    url = ('https://api.github.com/repos/sigmavirus24/github3.py/releases/'
           'assets/37944')
    with recorder.use_cassette('stream_response_to_file', **preserve_bytes):
        r = s.get(url, headers={'Accept': 'application/octet-stream'})
        path = stream.get_download_file_path(r, '/arbitrary/file.path')
        r.close()
        assert path == '/arbitrary/file.path'


def test_stream_response_to_file_uses_content_disposition():
    s = requests.Session()
    recorder = get_betamax(s)
    url = ('https://api.github.com/repos/sigmavirus24/github3.py/releases/'
           'assets/37944')
    filename = 'github3.py-0.7.1-py2.py3-none-any.whl'
    with recorder.use_cassette('stream_response_to_file', **preserve_bytes):
        r = s.get(url, headers={'Accept': 'application/octet-stream'},
                  stream=True)
        stream.stream_response_to_file(r)

    assert os.path.exists(filename)
    os.unlink(filename)


def test_stream_response_to_specific_filename():
    s = requests.Session()
    recorder = get_betamax(s)
    url = ('https://api.github.com/repos/sigmavirus24/github3.py/releases/'
           'assets/37944')
    filename = 'github3.py.whl'
    with recorder.use_cassette('stream_response_to_file', **preserve_bytes):
        r = s.get(url, headers={'Accept': 'application/octet-stream'},
                  stream=True)
        stream.stream_response_to_file(r, path=filename)

    assert os.path.exists(filename)
    os.unlink(filename)


def test_stream_response_to_directory():
    s = requests.Session()
    recorder = get_betamax(s)
    url = ('https://api.github.com/repos/sigmavirus24/github3.py/releases/'
           'assets/37944')

    td = tempfile.mkdtemp()
    try:
        filename = 'github3.py-0.7.1-py2.py3-none-any.whl'
        expected_path = os.path.join(td, filename)
        with recorder.use_cassette('stream_response_to_file', **preserve_bytes):
            r = s.get(url, headers={'Accept': 'application/octet-stream'},
                      stream=True)
            stream.stream_response_to_file(r, path=td)

        assert os.path.exists(expected_path)
    finally:
        shutil.rmtree(td)


def test_stream_response_to_existing_file():
    s = requests.Session()
    recorder = get_betamax(s)
    url = ('https://api.github.com/repos/sigmavirus24/github3.py/releases/'
           'assets/37944')
    filename = 'github3.py.whl'
    with open(filename, 'w') as f_existing:
        f_existing.write('test')

    with recorder.use_cassette('stream_response_to_file', **preserve_bytes):
        r = s.get(url, headers={'Accept': 'application/octet-stream'},
                  stream=True)
    try:
        stream.stream_response_to_file(r, path=filename)
    except stream.exc.StreamingError as e:
        assert str(e).startswith('File already exists:')
    else:
        assert False, "Should have raised a FileExistsError"
    finally:
        os.unlink(filename)


def test_stream_response_to_file_like_object():
    s = requests.Session()
    recorder = get_betamax(s)
    url = ('https://api.github.com/repos/sigmavirus24/github3.py/releases/'
           'assets/37944')
    file_obj = io.BytesIO()
    with recorder.use_cassette('stream_response_to_file', **preserve_bytes):
        r = s.get(url, headers={'Accept': 'application/octet-stream'},
                  stream=True)
        stream.stream_response_to_file(r, path=file_obj)

    assert 0 < file_obj.tell()


def test_stream_response_to_file_chunksize():
    s = requests.Session()
    recorder = get_betamax(s)
    url = ('https://api.github.com/repos/sigmavirus24/github3.py/releases/'
           'assets/37944')

    class FileWrapper(io.BytesIO):
        def __init__(self):
            super(FileWrapper, self).__init__()
            self.chunk_sizes = []

        def write(self, data):
            self.chunk_sizes.append(len(data))
            return super(FileWrapper, self).write(data)

    file_obj = FileWrapper()

    chunksize = 1231

    with recorder.use_cassette('stream_response_to_file', **preserve_bytes):
        r = s.get(url, headers={'Accept': 'application/octet-stream'},
                  stream=True)
        stream.stream_response_to_file(r, path=file_obj, chunksize=chunksize)

    assert 0 < file_obj.tell()

    assert len(file_obj.chunk_sizes) >= 1
    assert file_obj.chunk_sizes[0] == chunksize


@pytest.fixture
def streamed_response(chunks=None):
    chunks = chunks or [b'chunk'] * 8
    response = mock.MagicMock()
    response.raw.stream.return_value = chunks
    return response


def test_tee(streamed_response):
    response = streamed_response
    expected_len = len('chunk') * 8
    fileobject = io.BytesIO()
    assert expected_len == sum(len(c) for c in tee.tee(response, fileobject))
    assert fileobject.getvalue() == b'chunkchunkchunkchunkchunkchunkchunkchunk'


def test_tee_rejects_StringIO():
    fileobject = io.StringIO()
    with pytest.raises(TypeError):
        # The generator needs to be iterated over before the exception will be
        # raised
        sum(len(c) for c in tee.tee(None, fileobject))


def test_tee_to_file(streamed_response):
    response = streamed_response
    expected_len = len('chunk') * 8
    assert expected_len == sum(
        len(c) for c in tee.tee_to_file(response, 'tee.txt')
        )
    assert os.path.exists('tee.txt')
    os.remove('tee.txt')


def test_tee_to_bytearray(streamed_response):
    response = streamed_response
    arr = bytearray()
    expected_arr = bytearray(b'chunk' * 8)
    expected_len = len(expected_arr)
    assert expected_len == sum(
        len(c) for c in tee.tee_to_bytearray(response, arr)
        )
    assert expected_arr == arr


def test_tee_to_bytearray_only_accepts_bytearrays():
    with pytest.raises(TypeError):
        tee.tee_to_bytearray(None, object())



# -*- coding: utf-8 -*-
import unittest
import pytest

from requests_toolbelt import sessions
from requests import Request
from . import get_betamax


class TestBasedSession(unittest.TestCase):
    def test_request_with_base(self):
        session = sessions.BaseUrlSession('https://httpbin.org/')
        recorder = get_betamax(session)
        with recorder.use_cassette('simple_get_request'):
            response = session.get('/get')
        response.raise_for_status()

    def test_request_without_base(self):
        session = sessions.BaseUrlSession()
        with pytest.raises(ValueError):
            session.get('/')

    def test_request_override_base(self):
        session = sessions.BaseUrlSession('https://www.google.com')
        recorder = get_betamax(session)
        with recorder.use_cassette('simple_get_request'):
            response = session.get('https://httpbin.org/get')
        response.raise_for_status()
        assert response.json()['headers']['Host'] == 'httpbin.org'

    def test_prepared_request_with_base(self):
        session = sessions.BaseUrlSession('https://httpbin.org')
        request = Request(method="GET", url="/get")
        prepared_request = session.prepare_request(request)
        recorder = get_betamax(session)
        with recorder.use_cassette('simple_get_request'):
            response = session.send(prepared_request)
        response.raise_for_status()

    def test_prepared_request_without_base(self):
        session = sessions.BaseUrlSession()
        request = Request(method="GET", url="/")
        with pytest.raises(ValueError):
            prepared_request = session.prepare_request(request)
            session.send(prepared_request)

    def test_prepared_request_override_base(self):
        session = sessions.BaseUrlSession('https://www.google.com')
        request = Request(method="GET", url="https://httpbin.org/get")
        prepared_request = session.prepare_request(request)
        recorder = get_betamax(session)
        with recorder.use_cassette('simple_get_request'):
            response = session.send(prepared_request)
        response.raise_for_status()
        assert response.json()['headers']['Host'] == 'httpbin.org'



# -*- coding: utf-8 -*-
import io
import sys
import unittest
try:
    from unittest import mock
except ImportError:
    import mock
import pytest
import requests
from requests_toolbelt.multipart.decoder import BodyPart
from requests_toolbelt.multipart.decoder import (
    ImproperBodyPartContentException
)
from requests_toolbelt.multipart.decoder import MultipartDecoder
from requests_toolbelt.multipart.decoder import (
    NonMultipartContentTypeException
)
from requests_toolbelt.multipart.encoder import encode_with
from requests_toolbelt.multipart.encoder import MultipartEncoder


class TestBodyPart(unittest.TestCase):
    @staticmethod
    def u(content):
        major = sys.version_info[0]
        if major == 3:
            return content
        else:
            return unicode(content.replace(r'\\', r'\\\\'), 'unicode_escape')

    @staticmethod
    def bodypart_bytes_from_headers_and_values(headers, value, encoding):
        return b'\r\n\r\n'.join(
            [
                b'\r\n'.join(
                    [
                        b': '.join([encode_with(i, encoding) for i in h])
                        for h in headers
                    ]
                ),
                encode_with(value, encoding)
            ]
        )

    def setUp(self):
        self.header_1 = (TestBodyPart.u('Snowman'), TestBodyPart.u('☃'))
        self.value_1 = TestBodyPart.u('©')
        self.part_1 = BodyPart(
            TestBodyPart.bodypart_bytes_from_headers_and_values(
                (self.header_1,), self.value_1, 'utf-8'
            ),
            'utf-8'
        )
        self.part_2 = BodyPart(
            TestBodyPart.bodypart_bytes_from_headers_and_values(
                [], self.value_1, 'utf-16'
            ),
            'utf-16'
        )

    def test_equality_content_should_be_equal(self):
        part_3 = BodyPart(
            TestBodyPart.bodypart_bytes_from_headers_and_values(
                [], self.value_1, 'utf-8'
            ),
            'utf-8'
        )
        assert self.part_1.content == part_3.content

    def test_equality_content_equals_bytes(self):
        assert self.part_1.content == encode_with(self.value_1, 'utf-8')

    def test_equality_content_should_not_be_equal(self):
        assert self.part_1.content != self.part_2.content

    def test_equality_content_does_not_equal_bytes(self):
        assert self.part_1.content != encode_with(self.value_1, 'latin-1')

    def test_changing_encoding_changes_text(self):
        part_2_orig_text = self.part_2.text
        self.part_2.encoding = 'latin-1'
        assert self.part_2.text != part_2_orig_text

    def test_text_should_be_equal(self):
        assert self.part_1.text == self.part_2.text

    def test_no_headers(self):
        sample_1 = b'\r\n\r\nNo headers\r\nTwo lines'
        part_3 = BodyPart(sample_1, 'utf-8')
        assert len(part_3.headers) == 0
        assert part_3.content == b'No headers\r\nTwo lines'

    def test_no_crlf_crlf_in_content(self):
        content = b'no CRLF CRLF here!\r\n'
        with pytest.raises(ImproperBodyPartContentException):
            BodyPart(content, 'utf-8')


class TestMultipartDecoder(unittest.TestCase):
    def setUp(self):
        self.sample_1 = (
            ('field 1', 'value 1'),
            ('field 2', 'value 2'),
            ('field 3', 'value 3'),
            ('field 4', 'value 4'),
        )
        self.boundary = 'test boundary'
        self.encoded_1 = MultipartEncoder(self.sample_1, self.boundary)
        self.decoded_1 = MultipartDecoder(
            self.encoded_1.to_string(),
            self.encoded_1.content_type
        )

    def test_non_multipart_response_fails(self):
        jpeg_response = mock.NonCallableMagicMock(spec=requests.Response)
        jpeg_response.headers = {'content-type': 'image/jpeg'}
        with pytest.raises(NonMultipartContentTypeException):
            MultipartDecoder.from_response(jpeg_response)

    def test_length_of_parts(self):
        assert len(self.sample_1) == len(self.decoded_1.parts)

    def test_content_of_parts(self):
        def parts_equal(part, sample):
            return part.content == encode_with(sample[1], 'utf-8')

        parts_iter = zip(self.decoded_1.parts, self.sample_1)
        assert all(parts_equal(part, sample) for part, sample in parts_iter)

    def test_header_of_parts(self):
        def parts_header_equal(part, sample):
            return part.headers[b'Content-Disposition'] == encode_with(
                'form-data; name="{}"'.format(sample[0]), 'utf-8'
            )

        parts_iter = zip(self.decoded_1.parts, self.sample_1)
        assert all(
            parts_header_equal(part, sample)
            for part, sample in parts_iter
        )

    def test_from_response(self):
        response = mock.NonCallableMagicMock(spec=requests.Response)
        response.headers = {
            'content-type': 'multipart/related; boundary="samp1"'
        }
        cnt = io.BytesIO()
        cnt.write(b'\r\n--samp1\r\n')
        cnt.write(b'Header-1: Header-Value-1\r\n')
        cnt.write(b'Header-2: Header-Value-2\r\n')
        cnt.write(b'\r\n')
        cnt.write(b'Body 1, Line 1\r\n')
        cnt.write(b'Body 1, Line 2\r\n')
        cnt.write(b'--samp1\r\n')
        cnt.write(b'\r\n')
        cnt.write(b'Body 2, Line 1\r\n')
        cnt.write(b'--samp1--\r\n')
        response.content = cnt.getvalue()
        decoder_2 = MultipartDecoder.from_response(response)
        assert decoder_2.content_type == response.headers['content-type']
        assert (
            decoder_2.parts[0].content == b'Body 1, Line 1\r\nBody 1, Line 2'
        )
        assert decoder_2.parts[0].headers[b'Header-1'] == b'Header-Value-1'
        assert len(decoder_2.parts[1].headers) == 0
        assert decoder_2.parts[1].content == b'Body 2, Line 1'

    def test_from_responsecaplarge(self):
        response = mock.NonCallableMagicMock(spec=requests.Response)
        response.headers = {
            'content-type': 'Multipart/Related; boundary="samp1"'
        }
        cnt = io.BytesIO()
        cnt.write(b'\r\n--samp1\r\n')
        cnt.write(b'Header-1: Header-Value-1\r\n')
        cnt.write(b'Header-2: Header-Value-2\r\n')
        cnt.write(b'\r\n')
        cnt.write(b'Body 1, Line 1\r\n')
        cnt.write(b'Body 1, Line 2\r\n')
        cnt.write(b'--samp1\r\n')
        cnt.write(b'\r\n')
        cnt.write(b'Body 2, Line 1\r\n')
        cnt.write(b'--samp1--\r\n')
        response.content = cnt.getvalue()
        decoder_2 = MultipartDecoder.from_response(response)
        assert decoder_2.content_type == response.headers['content-type']
        assert (
            decoder_2.parts[0].content == b'Body 1, Line 1\r\nBody 1, Line 2'
        )
        assert decoder_2.parts[0].headers[b'Header-1'] == b'Header-Value-1'
        assert len(decoder_2.parts[1].headers) == 0
        assert decoder_2.parts[1].content == b'Body 2, Line 1'



# -*- coding: utf-8 -*-
"""Tests for the SocketOptionsAdapter and TCPKeepAliveAdapter."""
import contextlib
import platform
import socket
import sys

import pytest
try:
    from unittest import mock
except ImportError:
    import mock
import requests
from requests_toolbelt._compat import poolmanager

from requests_toolbelt.adapters import socket_options


@contextlib.contextmanager
def remove_keepidle():
    """A context manager to remove TCP_KEEPIDLE from socket."""
    TCP_KEEPIDLE = getattr(socket, 'TCP_KEEPIDLE', None)
    if TCP_KEEPIDLE is not None:
        del socket.TCP_KEEPIDLE

    yield

    if TCP_KEEPIDLE is not None:
        socket.TCP_KEEPIDLE = TCP_KEEPIDLE


@contextlib.contextmanager
def set_keepidle(value):
    """A context manager to set TCP_KEEPALIVE on socket always."""
    TCP_KEEPIDLE = getattr(socket, 'TCP_KEEPIDLE', None)
    socket.TCP_KEEPIDLE = value

    yield

    if TCP_KEEPIDLE is not None:
        socket.TCP_KEEPIDLE = TCP_KEEPIDLE
    else:
        del socket.TCP_KEEPIDLE


@mock.patch.object(requests, '__build__', 0x020500)
@mock.patch.object(poolmanager, 'PoolManager')
def test_options_passing_on_newer_requests(PoolManager):
    """Show that options are passed for a new enough version of requests."""
    fake_opts = [('test', 'options', 'fake')]
    adapter = socket_options.SocketOptionsAdapter(
        socket_options=fake_opts,
        pool_connections=10,
        pool_maxsize=5,
        pool_block=True,
    )
    PoolManager.assert_called_once_with(
        num_pools=10, maxsize=5, block=True,
        socket_options=fake_opts
    )
    assert adapter.socket_options == fake_opts


@mock.patch.object(requests, '__build__', 0x020300)
@mock.patch.object(poolmanager, 'PoolManager')
def test_options_not_passed_on_older_requests(PoolManager):
    """Show that options are not passed for older versions of requests."""
    fake_opts = [('test', 'options', 'fake')]
    socket_options.SocketOptionsAdapter(
        socket_options=fake_opts,
        pool_connections=10,
        pool_maxsize=5,
        pool_block=True,
    )
    assert PoolManager.called is False


@pytest.mark.xfail(sys.version_info.major == 2 and platform.system() == "Windows",
                   reason="Windows does not have TCP_KEEPINTVL in Python 2")
@mock.patch.object(requests, '__build__', 0x020500)
@mock.patch.object(poolmanager, 'PoolManager')
def test_keep_alive_on_newer_requests_no_idle(PoolManager):
    """Show that options are generated correctly from kwargs."""
    socket_opts = [
        (socket.IPPROTO_TCP, socket.TCP_NODELAY, 1),
        (socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1),
        (socket.IPPROTO_TCP, socket.TCP_KEEPINTVL, 10),
        (socket.IPPROTO_TCP, socket.TCP_KEEPCNT, 10),
    ]
    with remove_keepidle():
        adapter = socket_options.TCPKeepAliveAdapter(
            idle=30, interval=10, count=10,
            pool_connections=10,
            pool_maxsize=5,
            pool_block=True,
        )
    PoolManager.assert_called_once_with(
        num_pools=10, maxsize=5, block=True,
        socket_options=socket_opts
    )
    assert adapter.socket_options == socket_opts


@pytest.mark.xfail(sys.version_info.major == 2 and platform.system() == "Windows",
                   reason="Windows does not have TCP_KEEPINTVL in Python 2")
@mock.patch.object(requests, '__build__', 0x020500)
@mock.patch.object(poolmanager, 'PoolManager')
def test_keep_alive_on_newer_requests_with_idle(PoolManager):
    """Show that options are generated correctly from kwargs with KEEPIDLE."""
    with set_keepidle(3000):
        socket_opts = [
            (socket.IPPROTO_TCP, socket.TCP_NODELAY, 1),
            (socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1),
            (socket.IPPROTO_TCP, socket.TCP_KEEPINTVL, 10),
            (socket.IPPROTO_TCP, socket.TCP_KEEPCNT, 10),
            (socket.IPPROTO_TCP, socket.TCP_KEEPIDLE, 30),
        ]
        adapter = socket_options.TCPKeepAliveAdapter(
            idle=30, interval=10, count=10,
            pool_connections=10,
            pool_maxsize=5,
            pool_block=True,
        )

    PoolManager.assert_called_once_with(
        num_pools=10, maxsize=5, block=True,
        socket_options=socket_opts
    )
    assert adapter.socket_options == socket_opts



"""Test module for requests_toolbelt.utils.formdata."""
try:
    from urllib.parse import parse_qs
except ImportError:
    from urlparse import parse_qs

from requests_toolbelt.utils.formdata import urlencode

import pytest

dict_query = {
    'first_nested': {
        'second_nested': {
            'third_nested': {
                'fourth0': 'fourth_value0',
                'fourth1': 'fourth_value1',
            },
            'third0': 'third_value0',
        },
        'second0': 'second_value0',
    },
    'outter': 'outter_value',
}

list_query = [
    ('first_nested', [
        ('second_nested', [
            ('third_nested', [
                ('fourth0', 'fourth_value0'),
                ('fourth1', 'fourth_value1'),
            ]),
            ('third0', 'third_value0'),
        ]),
        ('second0', 'second_value0'),
    ]),
    ('outter', 'outter_value'),
]

mixed_dict_query = {
    'first_nested': {
        'second_nested': [
            ('third_nested', {
                'fourth0': 'fourth_value0',
                'fourth1': 'fourth_value1',
            }),
            ('third0', 'third_value0'),
        ],
        'second0': 'second_value0',
    },
    'outter': 'outter_value',
}

expected_parsed_query = {
    'first_nested[second0]': ['second_value0'],
    'first_nested[second_nested][third0]': ['third_value0'],
    'first_nested[second_nested][third_nested][fourth0]': ['fourth_value0'],
    'first_nested[second_nested][third_nested][fourth1]': ['fourth_value1'],
    'outter': ['outter_value'],
}


@pytest.mark.parametrize("query", [dict_query, list_query, mixed_dict_query])
def test_urlencode_flattens_nested_structures(query):
    """Show that when parsed, the structure is conveniently flat."""
    parsed = parse_qs(urlencode(query))

    assert parsed == expected_parsed_query


def test_urlencode_catches_invalid_input():
    """Show that queries are loosely validated."""
    with pytest.raises(ValueError):
        urlencode(['fo'])

    with pytest.raises(ValueError):
        urlencode([('foo', 'bar', 'bogus')])



import pytest
import requests

from requests_toolbelt.adapters import host_header_ssl as hhssl


@pytest.fixture
def session():
    """Create a session with our adapter mounted."""
    session = requests.Session()
    session.mount('https://', hhssl.HostHeaderSSLAdapter())


@pytest.mark.skip
class TestHostHeaderSSLAdapter(object):
    """Tests for our HostHeaderSNIAdapter."""

    def test_ssladapter(self, session):
        # normal mode
        r = session.get('https://example.org')
        assert r.status_code == 200

        # accessing IP address directly
        r = session.get('https://93.184.216.34',
                        headers={"Host": "example.org"})
        assert r.status_code == 200

        # vHost
        r = session.get('https://93.184.216.34',
                        headers={'Host': 'example.com'})
        assert r.status_code == 200

    def test_stream(self):
        self.session.get('https://54.175.219.8/stream/20',
                         headers={'Host': 'httpbin.org'},
                         stream=True)

    def test_case_insensitive_header(self):
        r = self.session.get('https://93.184.216.34',
                             headers={'hOSt': 'example.org'})
        assert r.status_code == 200

    def test_plain_requests(self):
        # test whether the reason for this adapter remains
        # (may be implemented into requests in the future)
        with pytest.raises(requests.exceptions.SSLError):
            requests.get(url='https://93.184.216.34',
                         headers={'Host': 'example.org'})



# -*- coding: utf-8 -*-
"""Test proxy digest authentication."""

import unittest
try:
    from unittest import mock
except ImportError:
    import mock

import requests
from requests_toolbelt.auth import http_proxy_digest


class TestProxyDigestAuth(unittest.TestCase):
    """Tests for the ProxyDigestAuth class."""

    def setUp(self):
        """Set up variables for each test."""
        self.username = "username"
        self.password = "password"
        self.auth = http_proxy_digest.HTTPProxyDigestAuth(
            self.username, self.password
        )
        self.prepared_request = requests.Request(
            'GET',
            'http://host.org/index.html'
        ).prepare()

    def test_with_existing_nonce(self):
        """Test if it will generate Proxy-Auth header when nonce present.

        Digest authentication's correctness will not be tested here.
        """
        self.auth.last_nonce = "bH3FVAAAAAAg74rL3X8AAI3CyBAAAAAA"
        self.auth.chal = {
            'nonce': self.auth.last_nonce,
            'realm': 'testrealm@host.org',
            'qop': 'auth'
        }

        # prepared_request headers should be clear before calling auth
        assert self.prepared_request.headers.get('Proxy-Authorization') is None
        self.auth(self.prepared_request)
        assert self.prepared_request.headers['Proxy-Authorization'] is not None

    def test_no_challenge(self):
        """Test that a response containing no auth challenge is left alone."""
        connection = MockConnection()
        first_response = connection.make_response(self.prepared_request)
        first_response.status_code = 404

        assert self.auth.last_nonce == ''
        final_response = self.auth.handle_407(first_response)
        headers = final_response.request.headers
        assert self.auth.last_nonce == ''
        assert first_response is final_response
        assert headers.get('Proxy-Authorization') is None

    def test_digest_challenge(self):
        """Test a response with a digest auth challenge causes a new request.

        This ensures that the auth class generates a new request with a
        Proxy-Authorization header.

        Digest authentication's correctness will not be tested here.
        """
        connection = MockConnection()
        first_response = connection.make_response(self.prepared_request)
        first_response.status_code = 407
        first_response.headers['Proxy-Authenticate'] = (
            'Digest'
            ' realm="Fake Realm", nonce="oS6WVgAAAABw698CAAAAAHAk/HUAAAAA",'
            ' qop="auth", stale=false'
        )

        assert self.auth.last_nonce == ''
        final_response = self.auth.handle_407(first_response)
        headers = final_response.request.headers
        assert self.auth.last_nonce != ''
        assert first_response is not final_response
        assert headers.get('Proxy-Authorization') is not None

    def test_ntlm_challenge(self):
        """Test a response without a Digest auth challenge is left alone."""
        connection = MockConnection()
        first_response = connection.make_response(self.prepared_request)
        first_response.status_code = 407
        first_response.headers['Proxy-Authenticate'] = 'NTLM'

        assert self.auth.last_nonce == ''
        final_response = self.auth.handle_407(first_response)
        headers = final_response.request.headers
        assert self.auth.last_nonce == ''
        assert first_response is final_response
        assert headers.get('Proxy-Authorization') is None


class MockConnection(object):
    """Fake connection object."""

    def send(self, request, **kwargs):
        """Mock out the send method."""
        return self.make_response(request)

    def make_response(self, request):
        """Make a response for us based on the request."""
        response = requests.Response()
        response.status_code = 200
        response.request = request
        response.raw = mock.MagicMock()
        response.connection = self
        return response

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""Tests for the AppEngineAdapter."""
import sys

try:
    from unittest import mock
except ImportError:
    import mock
import pytest
import requests

from requests_toolbelt import exceptions as exc

REQUESTS_SUPPORTS_GAE = requests.__build__ >= 0x021000

if REQUESTS_SUPPORTS_GAE:
    from requests.packages.urllib3.contrib import appengine as urllib3_appeng
    from requests_toolbelt.adapters import appengine
else:
    appengine = urllib3_appeng = None


@pytest.mark.skipif(sys.version_info >= (3,),
                    reason="App Engine doesn't support Python 3 (yet) and "
                           "urllib3's appengine contrib code is Python 2 "
                           "only. Until the latter changes, this test will "
                           "be skipped, unfortunately.")
@pytest.mark.skipif(not REQUESTS_SUPPORTS_GAE,
                    reason="Requires Requests v2.10.0 or later")
@mock.patch.object(urllib3_appeng, 'urlfetch')
def test_get(mock_urlfetch):
    """Tests a simple requests.get() call.

    App Engine urlfetch docs:
    https://cloud.google.com/appengine/docs/python/refdocs/google.appengine.api.urlfetch
    """
    response = mock.Mock(status_code=200, content='asdf', headers={})
    mock_urlfetch.fetch = mock.Mock(return_value=response)

    session = requests.Session()
    session.mount('http://', appengine.AppEngineAdapter())
    resp = session.get('http://url/', timeout=9, headers={'Foo': 'bar'})
    assert resp.status_code == 200
    assert resp.content == 'asdf'

    args, kwargs = mock_urlfetch.fetch.call_args
    assert args == ('http://url/',)
    assert kwargs['deadline'] == 9
    assert kwargs['headers']['Foo'] == 'bar'


@pytest.mark.skipif(sys.version_info >= (3,),
                    reason="App Engine doesn't support Python 3 (yet) and "
                           "urllib3's appengine contrib code is Python 2 "
                           "only. Until the latter changes, this test will "
                           "be skipped, unfortunately.")
@pytest.mark.skipif(not REQUESTS_SUPPORTS_GAE,
                    reason="Requires Requests v2.10.0 or later")
def test_appengine_monkeypatch():
    """Tests monkeypatching Requests adapters for AppEngine compatibility.
    """
    adapter = requests.sessions.HTTPAdapter

    appengine.monkeypatch()

    assert requests.sessions.HTTPAdapter == appengine.AppEngineAdapter
    assert requests.adapters.HTTPAdapter == appengine.AppEngineAdapter

    appengine.monkeypatch(validate_certificate=False)

    assert requests.sessions.HTTPAdapter == appengine.InsecureAppEngineAdapter
    assert requests.adapters.HTTPAdapter == appengine.InsecureAppEngineAdapter

    requests.sessions.HTTPAdapter = adapter
    requests.adapters.HTTPAdapter = adapter


@pytest.mark.skipif(sys.version_info >= (3,),
                    reason="App Engine doesn't support Python 3 (yet) and "
                           "urllib3's appengine contrib code is Python 2 "
                           "only. Until the latter changes, this test will "
                           "be skipped, unfortunately.")
@pytest.mark.skipif(not REQUESTS_SUPPORTS_GAE,
                    reason="Requires Requests v2.10.0 or later")
@mock.patch.object(urllib3_appeng, 'urlfetch')
def test_insecure_appengine_adapter(mock_urlfetch):
    adapter = appengine.InsecureAppEngineAdapter()

    assert not adapter._validate_certificate

    with pytest.warns(exc.IgnoringGAECertificateValidation):
        adapter = appengine.InsecureAppEngineAdapter(validate_certificate=True)



# -*- coding: utf-8 -*-
import requests
import unittest
import pytest

try:
    import OpenSSL
except ImportError:
    PYOPENSSL_AVAILABLE = False
else:
    PYOPENSSL_AVAILABLE = True
    from requests_toolbelt.adapters.x509 import X509Adapter
    from cryptography import x509
    from cryptography.hazmat.primitives.serialization import (
        Encoding,
        PrivateFormat,
        BestAvailableEncryption,
        load_pem_private_key,
    )
    import trustme

from requests_toolbelt import exceptions as exc
from . import get_betamax

REQUESTS_SUPPORTS_SSL_CONTEXT = requests.__build__ >= 0x021200

pytestmark = pytest.mark.filterwarnings(
    "ignore:'urllib3.contrib.pyopenssl' module is deprecated:DeprecationWarning")


class TestX509Adapter(unittest.TestCase):
    """Tests a simple requests.get() call using a .p12 cert.
    """
    def setUp(self):
        self.pkcs12_password_bytes = "test".encode('utf8')
        self.session = requests.Session()

    @pytest.mark.skipif(not REQUESTS_SUPPORTS_SSL_CONTEXT,
                        reason="Requires Requests v2.12.0 or later")
    @pytest.mark.skipif(not PYOPENSSL_AVAILABLE,
                        reason="Requires OpenSSL")
    def test_x509_pem(self):
        ca = trustme.CA()
        cert = ca.issue_cert(u'pkiprojecttest01.dev.labs.internal')
        cert_bytes = cert.cert_chain_pems[0].bytes()
        pk_bytes = cert.private_key_pem.bytes()

        adapter = X509Adapter(max_retries=3, cert_bytes=cert_bytes, pk_bytes=pk_bytes)
        self.session.mount('https://', adapter)
        recorder = get_betamax(self.session)
        with recorder.use_cassette('test_x509_adapter_pem'):
            r = self.session.get('https://pkiprojecttest01.dev.labs.internal/', verify=False)

        assert r.status_code == 200
        assert r.text

    @pytest.mark.skipif(not REQUESTS_SUPPORTS_SSL_CONTEXT,
                    reason="Requires Requests v2.12.0 or later")
    @pytest.mark.skipif(not PYOPENSSL_AVAILABLE,
                    reason="Requires OpenSSL")
    def test_x509_der_and_password(self):
        ca = trustme.CA()
        cert = ca.issue_cert(u'pkiprojecttest01.dev.labs.internal')
        cert_bytes = x509.load_pem_x509_certificate(
            cert.cert_chain_pems[0].bytes()).public_bytes(Encoding.DER)
        pem_pk = load_pem_private_key(cert.private_key_pem.bytes(), password=None)
        pk_bytes = pem_pk.private_bytes(Encoding.DER, PrivateFormat.PKCS8,
                                        BestAvailableEncryption(self.pkcs12_password_bytes))

        adapter = X509Adapter(max_retries=3, cert_bytes=cert_bytes, pk_bytes=pk_bytes,
                              password=self.pkcs12_password_bytes, encoding=Encoding.DER)
        self.session.mount('https://', adapter)
        recorder = get_betamax(self.session)
        with recorder.use_cassette('test_x509_adapter_der'):
            r = self.session.get('https://pkiprojecttest01.dev.labs.internal/', verify=False)

        assert r.status_code == 200
        assert r.text

    @pytest.mark.skipif(REQUESTS_SUPPORTS_SSL_CONTEXT, reason="Will not raise exc")
    def test_requires_new_enough_requests(self):
        with pytest.raises(exc.VersionMismatchError):
            X509Adapter()
